function [F] = getgradf(Z,N,M,Fval)

%
% **********************************
%
% gradient matrix calculator
%
% original code
% RAS 8/01
%
% **********************************

% forms gradient matrix for non-linear maximum likelihood
% routine MLEiter
%
% inputs are;
% (1) Z, vector with all data required to calculate F
% (2) N, #rows in F
% (3) M, #columns in F
% (4) Fval, an index describing the nature of the governing equation
%

% Fval=1 is a straight line equation
% Fval=2,3 is the crysal-melt partition coefficient equation
% Fval=4 is binary mixing
% Fval=5 is two-step diffusion
% Fval=6 is Ferguson and Church settling velocity equation
% Fval=7 is ternary mixing

if Fval==1

  F=zeros(N,M);
  term1=zeros(M,N);
  term2=zeros(N,N);
  term3=zeros(N,1);
  term4=zeros(M,N);
  term5=zeros(M,1);
  xpost=zeros(M,1);
  pert=zeros(M,1);
  df1=ones(N,1)*Z(2*N+1);
  df2=-ones(N,1);

  F(:,1:N)=diag(df1);
  F(:,N+1:2*N)=diag(df2);
  F(:,2*N+2)=ones(N,1);
  F(1:N,2*N+1)=Z(1:N);

elseif Fval==2

  F=zeros(N,M);

  Na=6.022e23;   % Avogadro's #
  R=8.314;      % Gas constant, g-moles-joules

  T=Z(2*N+4);   % temp for fit
  C=-4*pi*Na/(R*T);	% constant term
  ri=Z(N+4:2*N+3);
  r0=Z(N+3);
  E=Z(N+2);
  D0=Z(N+1);
  dr=ri-r0;
  dr2=ones(N,1)*r0^2-ri.^2;
  A=exp(C*E*((r0/2)*dr.^2+(dr.^3)/3))

  m1=-ones(N,1);
  F(:,1:N)=diag(m1);	% first N-columns are -1 on diagonal, df/dDi 
  F(:,N+1)=A;		% N+1 column is df/dD0
  F(:,N+2)=A*C*D0.*((r0/2)*dr.^2+(dr.^3)/3);	% N+2 column is df/dE
  F(:,N+3)=D0*C*E*A.*dr2/2;  % N+3 column is df/dr0

elseif Fval==3

  Na=6.022e23;   % Avogadro's #
  R=8.314;      % Gas constant, g-moles-joules

  T=Z(2*N+4);   % temp for fit
  C=-4*pi*Na/(R*T);     % constant term
  ri=Z(N+4:2*N+3);
  r0=Z(N+3);
  E=Z(N+2);
  D0=Z(N+1);
  Di=Z(1:N);
  dr=ri-r0;
  dr2=ones(N,1)*r0^2-ri.^2;
  fr=((r0/2)*dr.^2+(dr.^3)/3);

  m1=-ones(N,1)./Di;
  F(:,1:N)=diag(m1);    % first N-columns are -1 on diagonal, df/dDi
  F(:,N+1)=(1/D0)*ones(N,1);           % N+1 column is df/dD0
  F(:,N+2)=10^(-20)*C*fr;    % N+2 column is df/dE
  F(:,N+3)=10^(-20)*C*E*dr2/2;  % N+3 column is df/dr0

elseif Fval==4

  F=zeros(N,M);

  RXA=Z(2*N+1);
  RXB=Z(2*N+2);
  RYA=Z(2*N+3);
  RYB=Z(2*N+4);
  K=Z(2*N+5);
  RXM=Z(1:N);
  RYM=Z(N+1:2*N);

  rx=ones(N,1).*(RYB-RYA*K+(K-1)*RYM);
  ry=ones(N,1).*(RXA-RXB*K+(K-1)*RXM);
  F(:,1:N)=diag(rx);
  F(:,N+1:2*N)=diag(ry);
  F(:,2*N+1)=RYM-RYB;
  F(:,2*N+2)=K*(RYA-RYM);
  F(:,2*N+3)=K*(RXB-RXM);
  F(:,2*N+4)=RXM-RXA;
  F(:,2*N+5)=RXM.*RYM-RYA*RXM-RXB*RYM+RYA*RXB;

elseif Fval==5

  F=zeros(N,M);

  DA=Z(N+1);
  DB=Z(N+2);
  eA=Z(N+3);
  eB=Z(N+4);
  Temp=Z(N+5:2*N+4);
  R=Z(2*N+5);

  m1=-ones(N,1);
  F(:,1:N)=diag(m1);    % first N-columns are -1 on diagonal, df/dDi
  F(:,N+1)=exp(eA./(R*Temp));	% N+1 column is df/dD0A
  F(:,N+2)=exp(eB./(R*Temp));	% N+2 column is df/dD0B
  F(:,N+3)=(DA./(R*Temp)).*exp(eA./(R*Temp));	% N+3 column is df/DEa
  F(:,N+3)=(DB./(R*Temp)).*exp(eB./(R*Temp));	% N+4 column is df/Deb

elseif Fval==6

  R=2.61;	% specific gravity of clasts
  g=9.81;	% acceleration of gravity, m/s^2
  nu=1.83e-06;	% kinematic viscosity of seawater, m^2/s

  F=zeros(N,M);

  D=Z(1:N);
  w=Z(N+1:2*N);
  C1=Z(2*N+1);
  C2=Z(2*N+2);

  dfdD=(3/2)*sqrt(3/4*C2*R*g.*D)-2*R*g.*D./w;
  dfdw=R*g.*D.^2./(w.^2);
  dfdC1=nu;
  dfdC2=(1/2)*sqrt(3/4*R*g.*D.^3)*C2^(-1/2);

  F(:,1:N)=diag(dfdD);
  F(:,N+1:2*N)=diag(dfdw);
  F(:,2*N+1)=dfdC1;
  F(:,2*N+2)=dfdC2;

elseif Fval==7

  F=zeros(N,M);

  X1=Z(3*N+1);
  X2=Z(3*N+2);
  X3=Z(3*N+3);
  Y1=Z(3*N+4);
  Y2=Z(3*N+5);
  Y3=Z(3*N+6);
  Z1=Z(3*N+7);
  Z2=Z(3*N+8);
  Z3=Z(3*N+9);
  Ka2b1=Z(3*N+10);
  Kb1c2=Z(3*N+11);
  Ka1c3=Z(3*N+12);
  Ka1b3=Z(3*N+13);
  x=Z(1:N);
  y=Z(N+1:2*N);
  z=Z(2*N+1:3*N);

  A=Y2*Z1-Kb1c2*Y1*Z2+Ka2b1*Ka1c3*Y1*Z3-Ka1b3*Ka2b1*Y3*Z1+Ka1b3*Kb1c2*Y3*Z2-Ka1c3*Y2*Z3;
  B=X3*Z1-Ka1c3*X1*Z3+Ka1b3*Kb1c2*X1*Z2-Ka1b3*Ka2b1*X2*Z1+Ka2b1*Ka1c3*X2*Z3-Kb1c2*X3*Z2;
  C=X3*Y2-Ka1b3*Ka2b1*X2*Y3+Ka2b1*Ka1c3*X2*Y1-Ka1c3*X1*Y2+Ka1b3*Kb1c2*X1*Y3-Kb1c2*X3*Y1;
  D=Ka1c3*(1-Ka2b1)*Z3+Kb1c2*(1-Ka1b3)*Z2+(Ka1b3*Ka2b1-1)*Z1;
  E=Ka1b3*(Ka2b1-Kb1c2)*Y3+(Ka1c3-1)*Y2+(Kb1c2-Ka2b1*Ka1c3)*Y1;
  f=(Kb1c2-1)*X3+Ka2b1*(Ka1b3-Ka1c3)*X2+(Ka1c3-Ka1b3*Kb1c2)*X1;
  G=1-Ka1b3*Ka2b1+Kb1c2*(Ka1b3-1)+Ka1c3*(Ka2b1-1);
  H=Ka1c3*X1*Y2*Z3-X3*Y2*Z1+Ka1b3*Ka2b1*X2*Y3*Z1-Ka2b1*Ka1c3*X2*Y1*Z3+Kb1c2*X3*Y1*Z2-Ka1b3*Kb1c2*X1*Y3*Z2;

  rx=ones(N,1).*(A+D*y+E*z+G*y.*z);
  ry=ones(N,1).*(B+D*x+f*z+G*x.*z);
  rz=ones(N,1).*(C+E*x+f*y+G*x.*y);
  F(:,1:N)=diag(rx);
  F(:,N+1:2*N)=diag(ry);
  F(:,2*N+1:3*N)=diag(rz);
  F(:,3*N+1)=(Ka1b3*Kb1c2*Z2-Ka1c3*Z3)*y+(Ka1b3*Kb1c2*Y3-Ka1c3*Y2)*z+(Ka1c3-Ka1b3*Kb1c2)*y.*z+Ka1c3*Y2*Z3-Ka1b3*Kb1c2*Y3*Z2;
%fprintf('3N+1= %f\n',F(:,3*N+1))
  F(:,3*N+2)=(Ka2b1*Ka1c3*Z3-Ka1b3*Ka2b1*Z1)*y+(Ka2b1*Ka1c3*Y1-Ka1b3*Ka2b1*Y3)*z+(Ka2b1*(Ka1b3-Ka1c3))*y.*z+Ka2b1*(Ka1b3*Y3*Z1-Ka1c3*Y1*Z3);
%fprintf('3N+2= %f\n',F(:,3*N+2))
  F(:,3*N+3)=(Z1-Kb1c2*Z2)*y+(Y2-Kb1c2*Y1)*z+(Kb1c2-1)*y.*z+Kb1c2*Y1*Z2-Y2*Z1;
%fprintf('3N+3= %f\n',F(:,3*N+3))
  F(:,3*N+4)=(Ka2b1*Ka1c3*Z3-Kb1c2*Z2)*x+(Ka2b1*Ka1c3*X2-Kb1c2*X3)*z+(Kb1c2-Ka2b1*Ka1c3)*x.*z+Kb1c2*X3*Z2-Ka2b1*Ka1c3*X2*Z3;
%fprintf('3N+4= %f\n',F(:,3*N+4))
  F(:,3*N+5)=(Z1-Ka1c3*Z3)*x+(X3-Ka1c3*X1)*z+(Ka1c3-1)*x.*z+Ka1c3*X1*Z3-X3*Z1;
%fprintf('3N+5= %f\n',F(:,3*N+5))
  F(:,3*N+6)=(Ka1b3*(Kb1c2*Z2-Ka2b1*Z1))*x+(Ka1b3*(Kb1c2*X1-Ka2b1*X2))*z+(Ka1b3*(Ka2b1-Kb1c2))*x.*z+Ka1b3*(Ka2b1*X2*Z1-Kb1c2*X1*Z2);
%fprintf('3N+6= %f\n',F(:,3*N+6))
  F(:,3*N+7)=(Y2-Ka1b3*Ka2b1*Y3)*x+(X3-Ka1b3*Ka2b1*X2)*y+(Ka1b3*Ka2b1-1)*x.*y+Ka1b3*Ka2b1*X2*Y3-X3*Y2;
%fprintf('3N+7= %f\n',F(:,3*N+7))
  F(:,3*N+8)=(Kb1c2*(Ka1b3*Y3-Y1))*x+(Kb1c2*(Ka1b3*X1-X3))*y+(Kb1c2*(1-Ka1b3))*x.*y+Kb1c2*(X3*Y1-Ka1b3*X1*Y3);
%fprintf('3N+8= %f\n',F(:,3*N+8))
  F(:,3*N+9)=(Ka1c3*(Ka2b1*Y1-Y2))*x+(Ka1c3*(Ka2b1*X2-X1))*y+(Ka1c3*(1-Ka2b1))*x.*y+Ka1c3*(X1*Y2-Ka2b1*X2*Y1);
%fprintf('3N+9= %f\n',F(:,3*N+9))
  F(:,3*N+10)=(Ka1c3*Y1*Z3-Ka1b3*Y3*Z1)*x+(Ka1c3*X2*Z3-Ka1b3*X2*Z1)*y+(Ka1c3*X2*Y1-Ka1b3*X2*Y3)*z+(Ka1b3*Z1-Ka1c3*Z3)*x.*y+(Ka1b3*Y3-Ka1c3*Y1)*x.*z+(Ka1b3-Ka1c3)*X2*y.*z+(Ka1c3-Ka1b3)*x.*y.*z+Ka1b3*X2*Y3*Z1-Ka1c3*X2*Y1*Z3;
%fprintf('3N+10= %f\n',F(:,3*N+10))
  F(:,3*N+11)=(Ka1b3*Y3*Z2-Y1*Z2)*x+(Ka1b3*X1*Z2-X3*Z2)*y+(Ka1b3*X1*Y3-X3*Y1)*z+(1-Ka1b3)*Z2*x.*y+(Y1-Ka1b3*Y3)*x.*z+(X3-Ka1b3*X1)*y.*z+(Ka1b3-1)*x.*y.*z+X3*Y1*Z2-Ka1b3*X1*Y3*Z2;
%fprintf('3N+11= %f\n',F(:,3*N+11))
  F(:,3*N+12)=(Ka2b1*Y1*Z3-Y2*Z3)*x+(Ka2b1*X2*Z3-X1*Z3)*y+(Ka2b1*X2*Y1-X1*Y2)*z+(1-Ka2b1)*Z3*x.*y+(Y2-Ka2b1*Y1)*x.*z+(X1-Ka2b1*X2)*y.*z+(Ka2b1-1)*x.*y.*z+X1*Y2*Z3-Ka2b1*X2*Y1*Z3;
%fprintf('3N+12= %f\n',F(:,3*N+12))
  F(:,3*N+13)=(Kb1c2*Y3*Z2-Ka2b1*Y3*Z1)*x+(Kb1c2*X1*Z2-Ka2b1*X2*Z1)*y+(Kb1c2*X1*Y3-Ka2b1*X2*Y3)*z+(Ka2b1*Z1-Kb1c2*Z2)*x.*y+(Ka2b1*Y3-Kb1c2*Y3)*x.*z+(Ka2b1*X2-Kb1c2*X1)*y.*z+(Kb1c2-Ka2b1)*x.*y.*z+Ka2b1*X2*Y3*Z1-Kb1c2*X1*Y3*Z2;
%fprintf('3N+13= %f\n',F(:,3*N+13))

else

  error('Unrecognized Fval')

end
